# -----------------------------------------------------------------------------
# (C) Copyright Leandro Ponsoni
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of the copyright holder nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# 
#
# Plot Fig 9 from "An inter-comparison of the mass budget of the Arctic sea ice in 
# CMIP6 models", Keen et al (2021).
# XX denotes local pathnames/files etc required 

import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import read_budgetfiles as rb
import budget_utils as bu
from SIMIP_Massbudget_DataDic import DataDic # XX Dictionary defining all the filenames etc
from SIMIP_Massbudget_BudgetLineDic import BudgetLineDic # XX Dictionary defining budget line colours etc
import sys 
 
# Input data path
#-----------------
dataPath = '  ' # XX

# Model names as used in my DataDic
ModelNames_CMIP6 = ['NCAR_CESM2_CAM',   # CICE
          'NCAR_CESM2_WACCM', 		# CICE
          'NCC_NorESM2-LM',       	# CICE
          'NCC_NorESM2-MM',       	# CICE
          'MetO_UKESM1',      		# CICE (GSI8.1)
          'MetO_GC3.1_ORCA1',  		# CICE (GSI8.1)    
          'MetO_GC3.1_ORCA025',  	# CICE (GSI8.1)
          'CSIRO_ACCESS-CM2', 		# CICE (GSI8.1)
          'UCLouvain_EC-Earth3-Veg',    # LIM3
          'UCLouvain_EC-Earth3',        # LIM3
          'IPSL_CM6A-LR',          	# LIM3
          'CCCma_CanESM5_x1',          	# LIM2
          'GFDL-CM4',         		# SIS2
          'GFDL-ESM4',        		# SIS2
          'MRI_ESM2.0']         	# MRI

# Model names as required in the plot legend:
# These ones will be written into the .npz file,
# and need to match those in the plotting file
ModelNames_legend = ['NCAR_CESM2_CAM',   # CICE
          'NCAR_CESM2_WACCM', 		# CICE
          'NorESM2-LM',       		# CICE
          'NorESM2-MM',       		# CICE
          'UKESM1-0-LL',      		# CICE (GSI8.1)
          'HadGEM3-GC31-LL',  		# CICE (GSI8.1)    
          'HadGEM3-GC31-MM',  		# CICE (GSI8.1)
          'CSIRO-ACCESS-CM2', 		# CICE (GSI8.1)
          'EC-Earth3-Veg',    		# LIM3
          'EC-Earth3',        		# LIM3
          'IPSL_CM6A-LR',          	# LIM3
          'CanESM5',          		# LIM2
          'GFDL-CM4',         		# SIS2
          'GFDL-ESM4',        		# SIS2
          'MRI-ESM2']         		# MRI


ModelNames = ModelNames_CMIP6
plotTag = 'CMIP6_models'

nskip = 4

plotdir='Plot_files/'

#-----------------------------------------------

nmodels = len(ModelNames)
Var_labels = BudgetLineDic['ice']['Var_labels']
nvars = len(Var_labels)

massbudget_allmodels = np.empty([nmodels,nvars], dtype='float')


# Data processing ---------------------------------------------------------------
# Make the annual-mean ensemble-mean for each model, plus also the multi-model mean

for nmodel,ModelName in enumerate(ModelNames):
   #
   filepath = dataPath + DataDic[ModelName]['dir']
   Histfiles = DataDic[ModelName]['HistFiles_ice']
   nhistfiles = len(Histfiles)
   #
   # Make the evolution, but only use the Hist files
   years,icearea_mn,icearea_stdev,icemass_mn,icemass_stdev,massbudget_mn,massbudget_stdev = bu.annual_mn_evolution(
         filepath=filepath,histfiles=Histfiles,nhistfiles=nhistfiles,
         scenariofiles='Dummy',nscenariofiles=0,nskip=nskip)
   #
   yindex = np.where((years>=1960)) 
   massbudget_mn_sub = np.squeeze(massbudget_mn[yindex,:]) # Cut off any data before the start of the ref period
   massbudget_mn_ref = np.squeeze(np.mean(massbudget_mn_sub[0:29], axis=0))
   massbudget_mn_ref = massbudget_mn_ref/1.0e15 # Convert from kg to Gt x10^3 (per year)
   massbudget_allmodels[nmodel,:] = massbudget_mn_ref


# Writing out the data ----------------------------------------------------------------------------

Models = ModelNames

basal_growth_ens =  massbudget_allmodels[:,0]
frazil_ens       =  massbudget_allmodels[:,1]
top_melt_ens     =  massbudget_allmodels[:,2]
basal_melt_ens   =  massbudget_allmodels[:,3]
lateral_melt_ens =  massbudget_allmodels[:,4]
snowice_ens      =  massbudget_allmodels[:,5]
evapsubl_ens     =  massbudget_allmodels[:,6]
dynamics_ens     =  massbudget_allmodels[:,7]
total_ens        =  massbudget_allmodels[:,8]

# Saving a numpy .npz file 
file_npz = 'Data_files/mean_sea_ice_budget_terms_1960to89.npz'
np.savez(file_npz, Models           = ModelNames_legend,
                   basal_growth_ens = basal_growth_ens,
                   frazil_ens       = frazil_ens,
                   top_melt_ens     = top_melt_ens,
                   basal_melt_ens   = basal_melt_ens,
                   lateral_melt_ens = lateral_melt_ens,
                   snowice_ens      = snowice_ens,
                   evapsubl_ens     = evapsubl_ens,
                   dynamics_ens     = dynamics_ens,
                   total_ens        = total_ens)


# ---------------That's all folks --------------------


